function run_ssd(threshold, question)
% Based on work by 2017 Samuel Albanie
% Licensed under The MIT License [see LICENSE.md for details]

  if nargin < 1
      threshold = 0.6 ;
      question = 0 ;
  elseif nargin < 2
      threshold = 0.6 ;
  end

  opts.gpu = [] ;
  opts.modelPath = '' ;
  opts.wrapper = 'autonn' ;
  %opts = vl_argparse(opts, varargin) ;

  % The network is trained to prediction occurences
  % of the following classes from the pascal VOC challenge
  classes = {'background', 'aeroplane', 'bicycle', 'bird', ...
     'boat', 'bottle', 'bus', 'car', 'cat', 'chair', 'cow', 'diningtable', ...
     'dog', 'horse', 'motorbike', 'person', 'pottedplant', 'sheep', ...
     'sofa', 'train', 'tvmonitor'} ;

  % Load or download an example SSD model:
  modelName = 'ssd-mcn-pascal-vggvd-300.mat' ;
  paths = {opts.modelPath, ...
           modelName, ...
           fullfile(vl_rootnn, 'data/models', modelName), ...
           fullfile(vl_rootnn, 'data', 'models-import', modelName)} ;
  ok = find(cellfun(@(x) exist(x, 'file'), paths), 1) ;

  if isempty(ok)
    fprintf('Downloading the SSD model ... this may take a while\n') ;
    opts.modelPath = fullfile(vl_rootnn, 'data/models', modelName) ;
    mkdir(fileparts(opts.modelPath)) ;
    base = 'http://www.robots.ox.ac.uk/~albanie/models/ssd/%s' ;
    url = sprintf(base, modelName) ; urlwrite(url, opts.modelPath) ;
  else
    opts.modelPath = paths{ok} ;
  end

  % Load the network with the chosen wrapper
  net = loadModel(opts) ;

  % Load test images
  switch question
      case 0
          im_ids = randperm(300) ;
          im1 = single(imread(strcat('test_images/voc_images/', sprintf('%06d',im_ids(1)), '.jpg'))) ; 
          im1_resized = imresize(im1, net.meta.normalization.imageSize(1:2)) ;

          im2 = single(imread(strcat('test_images/voc_images/', sprintf('%06d',im_ids(2)), '.jpg'))) ; 
          im2_resized = imresize(im2, net.meta.normalization.imageSize(1:2)) ;
          
          im3 = single(imread(strcat('test_images/voc_images/', sprintf('%06d',im_ids(3)), '.jpg'))) ; 
          im3_resized = imresize(im3, net.meta.normalization.imageSize(1:2)) ;
          
          im4 = single(imread(strcat('test_images/voc_images/', sprintf('%06d',im_ids(4)), '.jpg'))) ; 
          im4_resized = imresize(im4, net.meta.normalization.imageSize(1:2)) ;
          imgCell={im1_resized,im2_resized,im3_resized,im4_resized};
          imgCellorig={im1,im2, im3, im4};    
      case 1
          %Photo credit: to burningman.org. Used under the fair use principles for transformative educational purposes.
          im1 = single(imread('test_images/img11.jpg')) ;
          im1_resized = imresize(im1, net.meta.normalization.imageSize(1:2)) ;
          imgCell={im1_resized};
          imgCellorig={im1};
      case 2
          %Photo credit: free wallpaper image. Used under the fair use principles for transformative educational purposes.
          im1 = single(imread('test_images/img21.jpg')) ; 
          im1_resized = imresize(im1, net.meta.normalization.imageSize(1:2)) ;

          %Photo credit: David Dodman, KNOM. Used under the fair use principles for transformative educational purposes.
          im2 = single(imread('test_images/img22.jpg')) ; 
          im2_resized = imresize(im2, net.meta.normalization.imageSize(1:2)) ;
          
          imgCell={im1_resized,im2_resized};
          imgCellorig={im1,im2};
      case 3
          %Photo credit: Duncan Rawlinson - Duncan.co photo from flickr.
          %Creative Commons license. https://creativecommons.org/licenses/by-nc/2.0/
          im1 = single(imread('test_images/img31.jpg')) ; 
          im1_resized = imresize(im1, net.meta.normalization.imageSize(1:2)) ;

          %Photo credit: Karri Huhtanen from flickr.
          %Creative Commons license. https://creativecommons.org/licenses/by-nc/2.0/
          im2 = single(imread('test_images/img32.jpg')) ; 
          im2_resized = imresize(im2, net.meta.normalization.imageSize(1:2)) ;
          
          imgCell={im1_resized,im2_resized};
          imgCellorig={im1,im2};
      case 4
          %Photo credit: Brad Templeton. Used under the fair use principles for transformative educational purposes.
          im1 = single(imread('test_images/img41.jpg')) ; 
          im1_resized = imresize(im1, net.meta.normalization.imageSize(1:2)) ;

          im2 = single(imread('test_images/img42.jpg')) ; 
          im2_resized = imresize(im2, net.meta.normalization.imageSize(1:2)) ;
          imgCell={im1_resized,im2_resized};
          imgCellorig={im1,im2};
      case 5
          %Photo credit: Susan Schmitz / shutterstock. Used under the fair use principles for transformative educational purposes.
          im1 = single(imread('test_images/img51.jpg')) ; 
          im1_resized = imresize(im1, net.meta.normalization.imageSize(1:2)) ;

          im2 = imrotate(im1, 180) ;
          im2_resized = imresize(im2, net.meta.normalization.imageSize(1:2)) ;
          imgCell={im1_resized,im2_resized};
          imgCellorig={im1,im2};
  end

  for i=1:length(imgCell)
  input_img = imgCell{i};
  % set inputs and run network
  switch opts.wrapper
    case 'dagnn' 
      net.eval({'data', input_img}) ;
      preds = net.vars(end).value ;
    case 'autonn'
      net.eval({'data', input_img}, 'test') ;
      preds = net.getValue('detection_out') ;
  end

  numKeep = length(find(preds(:, 2)>threshold)) ;

  [~, sortedIdx ] = sort(preds(:, 2), 'descend') ;
  preds = preds(sortedIdx, :) ;
  
  im = imgCellorig{i};

  % Extract the most confident predictions
  box = double(preds(1:numKeep,3:end)) ;
  confidence = preds(1:numKeep,2) ;
  label = classes(preds(1:numKeep,1)) ;

  % Return image to cpu for visualisation
  if numel(opts.gpu) > 0, im = gather(im) ; end

  % Diplay prediction at the orginal scale
  figure ; 
  im = im / 255 ; CM = spring(numKeep); 
  x = box(:,1) * size(im, 2) ; y = box(:,2) * size(im, 1) ;
  width = box(:,3) * size(im, 2) - x ; height = box(:,4) * size(im, 1) - y ;
  rectangle = [x y width height];
  im = insertShape(im, 'Rectangle', rectangle, 'LineWidth', 4, ...
                     'Color', CM(1:numKeep,:)) ;
  imagesc(im) ;
  for ii = 1:numKeep
    str = sprintf('%s: %.2f', label{ii}, confidence(ii)) ;
    text(x(ii), y(ii)-10, str, 'FontSize', 14, ...
        'BackgroundColor', CM(ii,:)) ;
  end
  title(sprintf('Detection results at threshold: %.2f', threshold), ...
                   'FontSize', 15) ;
  axis off ;
  end

% ----------------------------
function net = loadModel(opts)
% ----------------------------
  net = load(opts.modelPath) ; 
  if ~isfield(net, 'forward') % dagnn loader
    net = dagnn.DagNN.loadobj(net) ;
    switch opts.wrapper
      case 'dagnn' 
        net.mode = 'test' ; 
      case 'autonn'
        out = Layer.fromDagNN(net, @extras_autonn_custom_fn) ; 
        net = Net(out{:}) ;
    end
  else % load directly using autonn
    net = Net(net) ;
  end
